package main

import (
	"context"
	"github.com/disgoorg/disgo"
	"github.com/disgoorg/disgo/bot"
	"github.com/disgoorg/disgo/cache"
	"github.com/disgoorg/disgo/gateway"
	"github.com/disgoorg/snowflake/v2"
	"github.com/getsentry/sentry-go"
	"log"
	"os"
	"os/signal"
	"syscall"
	"time"
)

var (
	token           = os.Getenv("disgo_token")
	registerGuildID = snowflake.GetEnv("disgo_guild_id")
	channelTempID   snowflake.ID
)

// sentry
var release string

func main() {

	err := sentry.Init(sentry.ClientOptions{
		// Either set your DSN here or set the SENTRY_DSN environment variable.
		//Dsn: "",
		// Set TracesSampleRate to 1.0 to capture 100%
		// of transactions for performance monitoring.
		// We recommend adjusting this value in production,
		TracesSampleRate: 1.0,
		Release:          release,
	})
	if err != nil {
		log.Fatalf("sentry.Init: %s", err)
	}
	// Flush buffered events before the program terminates.
	defer sentry.Flush(2 * time.Second)

	log.Printf("INFO: starting tempbot...")
	log.Printf("INFO: disgo version: %v", disgo.Version)

	// permissions: Manage Messages
	// intent:
	client, err := disgo.New(token,
		bot.WithGatewayConfigOpts(
			gateway.WithIntents(gateway.IntentsNone),
			gateway.WithAutoReconnect(true),
		),
		bot.WithCacheConfigOpts(
			cache.WithCaches(
				cache.FlagsNone,
			),
		),
	)
	if err != nil {
		log.Fatal("error while building disgo instance: ", err)
		return
	}

	defer client.Close(context.TODO())

	if err = client.OpenGateway(context.TODO()); err != nil {
		log.Fatal("error while connecting to gateway: ", err)
	}

	channels, err := client.Rest().GetGuildChannels(registerGuildID)
	for _, channel := range channels {
		switch channel.Name() {
		case "⌛-temp":
			channelTempID = channel.ID()
		}
	}
	if channelTempID == 0 {
		log.Fatal("couldn't find needed channel")
	}

	ticker := time.NewTicker(1 * time.Minute)
	quit := make(chan struct{})
	go func() {
		for {
			select {
			case <-ticker.C:
				messages, err := client.Rest().GetMessages(channelTempID, 0, 0, 0, 100)
				if err != nil {
					log.Printf("ERROR: getting messages: %v", err)
				}
				var messageIDs []snowflake.ID
				for _, message := range messages {
					if message.CreatedAt.Add(30 * time.Minute).Before(time.Now()) {
						messageIDs = append(messageIDs, message.ID)
					}
				}
				log.Printf("DEBUG: deleting messages: %v", len(messageIDs))
				if len(messageIDs) == 1 {
					err = client.Rest().DeleteMessage(channelTempID, messageIDs[0])
					if err != nil {
						log.Printf("ERROR: deleting messages: %v", err)
					}
				}
				if len(messageIDs) > 1 {
					err = client.Rest().BulkDeleteMessages(channelTempID, messageIDs)
					if err != nil {
						log.Printf("ERROR: deleting messages: %v", err)
					}
				}
			case <-quit:
				ticker.Stop()
				return
			}
		}
	}()

	log.Printf("INFO: tempbot is now running. Press CTRL-C to exit.")
	sentry.ConfigureScope(func(scope *sentry.Scope) {
		scope.SetLevel(sentry.LevelDebug)
		sentry.CaptureMessage("DEBUG: tempbot started")
	})

	s := make(chan os.Signal, 1)
	signal.Notify(s, syscall.SIGINT, syscall.SIGTERM, os.Interrupt)
	<-s
}
